@@ -25,10 +25,13 @@
 "print.tgp" <-
 function(x, ...)
 {
-	cat("This is a 'tgp' class object\n")
-	cat("It is basically a list with the following entries:\n\n")
-	print(names(x), quote=FALSE)
+  cat("\nThis is a 'tgp' class object. ",
+      "It is basically a list with the following entries:\n", fill=TRUE)
+  print(names(x), quote=FALSE)
 
-	cat("\nSee tgp or btgp for an explanation of the individual entries\n")
-	cat("See plot.tgp and tgp.trees for help with visualization\n")
+  cat("\nSee ?btgp for an explanation of the individual entries. ",
+      "See plot.tgp and tgp.trees for help with visualization.\n", fill=TRUE)
+  
+  cat("The $trace field, if it exists, is of class 'tgptraces'",
+      "and has its own print statement\n", fill=TRUE)
 }
