@@ -35,7 +35,7 @@ class Tree;
 #define BUFFMAX 256
 
 // These are already defined in GP.h
-//  typedef enum BETA_PRIOR {B0=801, BMLE=802, BFLAT=803, BCART=804, B0TAU=805} BETA_PRIOR;
+//  typedef enum BETA_PRIOR {B0=801, BMLE=802, BFLAT=803, B0NOT=804, BMZT=805} BETA_PRIOR;
 
 
 class MrGp : public Base
@@ -113,6 +113,7 @@ class MrGp_Prior : public Base_Prior
 {
  private:
 
+  bool cart;                 /* indicates if only b[0]=mu should be used */
   BETA_PRIOR beta_prior;	/* indicator for type of Beta Prior */  
   Corr_Prior *corr_prior;
   unsigned int col;		
@@ -188,6 +189,7 @@ class MrGp_Prior : public Base_Prior
   double** get_Ti(void);
   double* get_b0(void);
   double R(void);
+  bool Cart(void);
 
   Corr_Prior* CorrPrior(void);
   BETA_PRIOR BetaPrior(void);
